@@ -1,5 +1,5 @@
 /* dvdcpy - A DVD backup tool
- * Copyright (C) 2004-2008 Olivier Rolland <billl@users.sf.net>
+ * Copyright (C) 2004-2009 Olivier Rolland <billl@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,21 +19,22 @@
 #define _LARGEFILE64_SOURCE
 #define _FILE_OFFSET_BITS 64
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
 #include <getopt.h>
 #include <stdarg.h>
-#include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
 #include <sys/stat.h>
 
-#ifdef HAVE_STATVFS
+#ifdef HAVE_SYS_STATVFS_H
 #include <sys/statvfs.h>
 #endif
 
@@ -133,7 +134,10 @@ dvd_file_size (dvd_reader_t *reader, unsigned int vts, dvd_read_domain_t domain)
   dvd_stat_t statbuf;
 
   if (DVDFileStat (reader, vts, domain, &statbuf) < 0)
+  {
+    fprintf (stderr, "Error: Cannot stat VOB for VTS %u\n", vts);
     return -1;
+  }
 
   return statbuf.size / (uint32_t) DVD_VIDEO_LB_LEN;
 }
@@ -298,7 +302,7 @@ dvd_copy_blocks (dvd_file_t *file, size_t offset, size_t size, int fd)
 {
   unsigned char buffer[512 * DVD_VIDEO_LB_LEN];
   unsigned int start;
-  size_t count, len;
+  ssize_t count, len;
 
   for (count = 512, start = offset; offset < start + size; offset += count)
   {
@@ -307,7 +311,7 @@ dvd_copy_blocks (dvd_file_t *file, size_t offset, size_t size, int fd)
     len = count;
 
     count = DVDReadBlocks (file, offset, len, buffer);
-    if (!count)
+    if (count < 0)
     {
       fprintf (stderr, "Error: Cannot read data\n");
       return -1;
@@ -412,9 +416,13 @@ dvd_copy_ifo (dvd_reader_t *reader, unsigned int vts, const char *output)
   int fd;
 
   dvd_file_t *file;
+  dvd_stat_t dvd_stat;
 
-  if ((size = dvd_file_size (reader, vts, DVD_READ_INFO_FILE)) <= 0)
-    return size;
+  if (DVDFileStat (reader, vts, DVD_READ_INFO_FILE, &dvd_stat) < 0)
+  {
+    fprintf (stderr, "Error: Cannot stat IFO for VTS %u\n", vts);
+    return -1;
+  }
 
   if (vts == 0)
     snprintf (filename, FILENAME_MAX, "%s/VIDEO_TS/VIDEO_TS.IFO", output);
@@ -447,6 +455,8 @@ dvd_copy_ifo (dvd_reader_t *reader, unsigned int vts, const char *output)
     return -1;
   }
 
+  size = dvd_stat.size / (uint32_t) DVD_VIDEO_LB_LEN;
+
   fprintf (stdout, "Copying IFO for VTS %u\n", vts);
   copied = dvd_copy_blocks (file, 0, size, fd);
   fprintf (stdout, "\n");
@@ -456,7 +466,10 @@ dvd_copy_ifo (dvd_reader_t *reader, unsigned int vts, const char *output)
   close (fd);
 
   if (copied != size)
+  {
+    fprintf (stderr, "Error: Failed to copy IFO for VTS %u\n", vts);
     return -1;
+  }
 
   return size;
 }
@@ -514,7 +527,10 @@ dvd_copy_bup (dvd_reader_t *reader, unsigned int vts, const char *output)
   close (fd);
 
   if (copied != size)
+  {
+    fprintf (stderr, "Error: Failed to copy BUP for VTS %u\n", vts);
     return -1;
+  }
 
   return size;
 }
@@ -572,7 +588,10 @@ dvd_copy_menu (dvd_reader_t *reader, unsigned int vts, const char *output)
   close (fd);
 
   if (copied != size)
+  {
+    fprintf (stderr, "Error: Failed to copy menu for VTS %u\n", vts);
     return -1;
+  }
 
   return size;
 }
@@ -582,8 +601,8 @@ dvd_copy_vob (dvd_reader_t *reader, unsigned int vts, const char *output)
 {
   char filename[FILENAME_MAX];
   struct stat buf;
-  ssize_t size;
-  size_t copied;
+  ssize_t size, copied;
+  size_t offset;
   int vob, fd;
 
   dvd_file_t *dvd_file;
@@ -602,7 +621,7 @@ dvd_copy_vob (dvd_reader_t *reader, unsigned int vts, const char *output)
     return -1;
   }
 
-  for (vob = 0, copied = 0; vob < dvd_stat.nr_parts; vob ++)
+  for (vob = 0, offset = 0; vob < dvd_stat.nr_parts; vob ++)
   {
     snprintf (filename, FILENAME_MAX, "%s/VIDEO_TS/VTS_%02u_%u.VOB", output, vts, vob + 1);
 
@@ -631,26 +650,29 @@ dvd_copy_vob (dvd_reader_t *reader, unsigned int vts, const char *output)
     size = dvd_stat.parts_size[vob] / DVD_VIDEO_LB_LEN;
 
     fprintf (stdout, "Copying VOB %u for VTS %u\n", vob + 1, vts);
-    if ((size = dvd_copy_blocks (dvd_file, copied, size, fd)) < 0)
-    {
-      DVDCloseFile (dvd_file);
-      close (fd);
-
-      return -1;
-    }
+    copied = dvd_copy_blocks (dvd_file, offset, size, fd);
     fprintf (stdout, "\n");
 
-    copied += size;
-
     close (fd);
+
+    if (copied != size)
+    {
+      fprintf (stderr, "Error: Failed to copy VOB part %u for VTS %u\n", vob, vts);
+      break;
+    }
+
+    offset += size;
   }
 
-  DVDCloseFile (dvd_file);
+      DVDCloseFile (dvd_file);
 
-  if (copied != dvd_stat.size / (uint32_t) DVD_VIDEO_LB_LEN)
-    return -1;
+  if (offset != dvd_stat.size / (uint32_t) DVD_VIDEO_LB_LEN)
+  {
+    fprintf (stderr, "Error: Failed to copy VOB for VTS %u\n", vts);
+      return -1;
+    }
 
-  return copied;
+  return offset;
 }
 
 static ssize_t
@@ -845,7 +867,6 @@ main (int argc, char *argv[])
 
   if ((dvd_copy (reader, vmg_file, vmg, vtss, output)) < 0)
   {
-    /* fprintf (stderr, "Error: Failed to copy the DVD\n"); */
     ifoClose (vmg_file);
     DVDClose (reader);
     return EXIT_FAILURE;
