@@ -1,5 +1,5 @@
 /* OGMRip - A library for DVD ripping and encoding
- * Copyright (C) 2004-2008 Olivier Rolland <billl@users.sf.net>
+ * Copyright (C) 2004-2009 Olivier Rolland <billl@users.sourceforge.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -678,7 +678,7 @@ ogmrip_subp_file_new_sub (const gchar *filename, GError **error)
 
   gchar *sr[] = 
   {
-    "microdvd" "subrip", "subviewer", "sami", "vplayer",
+    "microdvd", "subrip", "subviewer", "sami", "vplayer",
     "rt", "ssa", "pjs", "mpsub", "aqt", "subviewer 2.0",
     "subrip 0.9", "jacosub", "mpl2", NULL
   };
@@ -710,7 +710,7 @@ ogmrip_subp_file_new_sub (const gchar *filename, GError **error)
     if (g_str_has_prefix (lines[i], "SUB: ") &&
         g_str_has_prefix (lines[i] + 5, "Detected subtitle file format: "))
     {
-      for (j = 0; sr[j]; j++)
+      for (j = 0; sr[j] && format < 0; j++)
         if (strcmp (lines[i] + 36, sr[j]) == 0)
           format = OGMRIP_FORMAT_MICRODVD + j;
     }
