@@ -1,5 +1,5 @@
 /* OGMRip - A library for DVD ripping and encoding
- * Copyright (C) 2004-2008 Olivier Rolland <billl@users.sf.net>
+ * Copyright (C) 2004-2009 Olivier Rolland <billl@users.sourceforge.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -254,6 +254,23 @@ ogmrip_fs_mkftemp (const gchar *tmpl, GError **error)
   return name;
 }
 
+#ifndef HAVE_MKDTEMP
+static gchar *
+mkdtemp (gchar *template)
+{
+  gchar *path;
+
+  path = mktemp (template);
+  if (path == NULL || path[0] == '\0')
+    return NULL;
+
+  if (mkdir (path, 0700) < 0)
+    return NULL;
+
+  return path;
+}
+#endif
+
 /**
  * ogmrip_fs_mkdtemp:
  * @tmpl: Template for directory name, basename only
@@ -657,3 +674,30 @@ ogmrip_fs_set_extension (const gchar *filename, const gchar *extension)
   }
 }
 
+gchar *
+ogmrip_fs_get_full_path (const gchar *filename)
+{
+  gchar *fullname, *dirname, *basename, *cwd;
+
+  g_return_val_if_fail (filename != NULL, NULL);
+
+  if (g_path_is_absolute (filename))
+    return g_strdup (filename);
+
+  cwd = g_get_current_dir ();
+
+  dirname = g_path_get_dirname (filename);
+  g_chdir (dirname);
+  g_free (dirname);
+
+  dirname = g_get_current_dir ();
+  g_chdir (cwd);
+  g_free (cwd);
+
+  basename = g_path_get_basename (filename);
+  fullname = g_build_filename (dirname, basename, NULL);
+  g_free (basename);
+
+  return fullname;
+}
+
