@@ -1,5 +1,5 @@
 /* OGMRip - A library for DVD ripping and encoding
- * Copyright (C) 2004-2008 Olivier Rolland <billl@users.sf.net>
+ * Copyright (C) 2004-2009 Olivier Rolland <billl@users.sourceforge.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -266,7 +266,7 @@ ogmrip_mplayer_wav_command (OGMRipAudioCodec *audio, gboolean header, const gcha
   }
 
   g_ptr_array_add (argv, g_strdup ("-vc"));
-  g_ptr_array_add (argv, g_strdup ("null"));
+  g_ptr_array_add (argv, g_strdup ("dummy"));
   g_ptr_array_add (argv, g_strdup ("-vo"));
   g_ptr_array_add (argv, g_strdup ("null"));
   g_ptr_array_add (argv, g_strdup ("-ao"));
@@ -391,11 +391,20 @@ ogmrip_mplayer_wav_command (OGMRipAudioCodec *audio, gboolean header, const gcha
 gdouble
 ogmrip_mplayer_wav_watch (OGMJobExec *exec, const gchar *buffer, OGMRipAudioCodec *audio)
 {
-  gchar a[10], v[10], av[15], ct[15];
-  gint frame, decoded;
+  gchar pos[12], pos_time[12], total[12];
+  static gdouble start;
+  gdouble secs;
 
-  if (sscanf (buffer, "A:%s V:%s A-V:%s ct:%s %d/%d", a, v, av, ct, &decoded, &frame) == 6)
-    return decoded / (gdouble) ogmrip_codec_get_frames (OGMRIP_CODEC (audio));
+  if (g_str_equal (buffer, "Starting playback..."))
+    start = 0;
+  else if (sscanf (buffer, "A: %s %s of %s", pos, pos_time, total) == 3)
+  {
+    secs = strtod (pos, NULL);
+    if (!start)
+      start = secs;
+
+    return (secs - start) / ogmrip_codec_get_length (OGMRIP_CODEC (audio), NULL);
+  }
 
   return -1.0;
 }
