@@ -1,5 +1,5 @@
 /* OGMRip - A library for DVD ripping and encoding
- * Copyright (C) 2004-2008 Olivier Rolland <billl@users.sf.net>
+ * Copyright (C) 2004-2009 Olivier Rolland <billl@users.sourceforge.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -43,6 +43,8 @@ enum
   PROP_0,
   PROP_B_PYRAMID,
   PROP_BRDO,
+  PROP_PSY_RD,
+  PROP_PSY_TRELLIS,
   PROP_CABAC,
   PROP_FRAMEREF,
   PROP_LEVEL_IDC,
@@ -84,6 +86,8 @@ struct _OGMRipX264Priv
   gboolean weight_b;
   gboolean x88dct;
   gboolean global_header;
+  gdouble psy_rd;
+  gdouble psy_trellis;
 };
 
 static const gchar *me_name[] =
@@ -91,6 +95,10 @@ static const gchar *me_name[] =
   NULL, "dia", "hex", "umh", "esa"
 };
 
+gboolean x264_have_brdo = FALSE;
+gboolean x264_have_bime = FALSE;
+gboolean x264_have_psy  = FALSE;
+
 static gint
 ogmrip_x264_get_crf (OGMRipVideoCodec *video)
 {
@@ -116,6 +124,7 @@ ogmrip_x264_command (OGMRipVideoCodec *video, guint pass, guint passes, const gc
 
   const gchar *output;
   gint quality, bitrate, vid, threads, bframes;
+  gboolean trellis;
 
   g_return_val_if_fail (OGMRIP_IS_VIDEO_CODEC (video), NULL);
 
@@ -147,7 +156,7 @@ ogmrip_x264_command (OGMRipVideoCodec *video, guint pass, guint passes, const gc
 
   if (MPLAYER_CHECK_VERSION (1,0,1,0))
   {
-    if (x264->priv->brdo)
+    if (x264_have_brdo && x264->priv->brdo)
       g_string_append (options, ":brdo");
     if (x264->priv->x88dct)
       g_string_append (options, ":8x8dct");
@@ -184,7 +193,8 @@ ogmrip_x264_command (OGMRipVideoCodec *video, guint pass, guint passes, const gc
       g_string_append (options, ":4x4mv");
   }
 
-  if (ogmrip_video_codec_get_trellis (video))
+  trellis = ogmrip_video_codec_get_trellis (video);
+  if (trellis)
   {
     if (quality == OGMRIP_QUALITY_EXTREME)
       g_string_append (options, ":trellis=2");
@@ -197,9 +207,19 @@ ogmrip_x264_command (OGMRipVideoCodec *video, guint pass, guint passes, const gc
   bframes = ogmrip_video_codec_get_max_b_frames (video);
   g_string_append_printf (options, ":bframes=%d", bframes);
 
-  if (bframes > 0)
+  if (x264_have_bime && bframes > 0)
     g_string_append (options, ":bime");
 
+  if (x264_have_psy)
+  {
+    gchar psy_rd[G_ASCII_DTOSTR_BUF_SIZE], psy_trellis[G_ASCII_DTOSTR_BUF_SIZE];
+
+    g_ascii_formatd (psy_rd, G_ASCII_DTOSTR_BUF_SIZE, "%.1f", x264->priv->subq >= 6 ? x264->priv->psy_rd : 0.0);
+    g_ascii_formatd (psy_trellis, G_ASCII_DTOSTR_BUF_SIZE, "%.1f", trellis ? x264->priv->psy_trellis : 0.0);
+
+    g_string_append_printf (options, ":psy-rd=%s,%s", psy_rd, psy_trellis);
+  }
+
   bitrate = ogmrip_video_codec_get_bitrate (video);
   if (bitrate > 0)
     g_string_append_printf (options, ":bitrate=%u", bitrate / 1000);
@@ -225,7 +245,7 @@ ogmrip_x264_command (OGMRipVideoCodec *video, guint pass, guint passes, const gc
   
   threads = ogmrip_video_codec_get_threads (video);
   if (threads > 0)
-    g_string_append_printf (options, ":threads=%u", CLAMP (threads, 1, 4));
+    g_string_append_printf (options, ":threads=%u", CLAMP (threads, 1, 16));
   else
     g_string_append (options, ":threads=auto");
 
@@ -276,7 +296,7 @@ ogmrip_x264_class_init (OGMRipX264Class *klass)
 
   g_object_class_install_property (gobject_class, PROP_LEVEL_IDC, 
         g_param_spec_uint ("level_idc", "Level IDC property", "Set level IDC", 
-           10, 51, 51, G_PARAM_READWRITE));
+           0, 51, 51, G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, PROP_ME, 
         g_param_spec_uint ("me", "Motion estimation property", "Set motion estimation", 
@@ -298,9 +318,21 @@ ogmrip_x264_class_init (OGMRipX264Class *klass)
         g_param_spec_boolean ("b_pyramid", "B pyramid property", "Set b pyramid", 
            FALSE, G_PARAM_READWRITE));
 
-  g_object_class_install_property (gobject_class, PROP_BRDO, 
-        g_param_spec_boolean ("brdo", "Brdo property", "Set brdo", 
-           FALSE, G_PARAM_READWRITE));
+  if (x264_have_brdo)
+    g_object_class_install_property (gobject_class, PROP_BRDO, 
+          g_param_spec_boolean ("brdo", "Brdo property", "Set brdo", 
+             FALSE, G_PARAM_READWRITE));
+
+  if (x264_have_psy)
+  {
+    g_object_class_install_property (gobject_class, PROP_PSY_RD, 
+          g_param_spec_double ("psy_rd", "Psy RD property", "Set psy-rd", 
+             0.0, G_MAXDOUBLE, 1.0, G_PARAM_READWRITE));
+
+    g_object_class_install_property (gobject_class, PROP_PSY_TRELLIS, 
+          g_param_spec_double ("psy_trellis", "Psy trellis property", "Set psy-trellis", 
+             0.0, G_MAXDOUBLE, 0.0, G_PARAM_READWRITE));
+  }
 
   g_object_class_install_property (gobject_class, PROP_CABAC, 
         g_param_spec_boolean ("cabac", "Cabac property", "Set cabac", 
@@ -380,6 +412,12 @@ ogmrip_x264_get_property (GObject *gobject, guint property_id, GValue *value, GP
     case PROP_GLOBAL_HEADER:
       g_value_set_boolean (value, x264->priv->global_header);
       break;
+    case PROP_PSY_RD:
+      g_value_set_double (value, x264->priv->psy_rd);
+      break;
+    case PROP_PSY_TRELLIS:
+      g_value_set_double (value, x264->priv->psy_trellis);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
       break;
@@ -434,6 +472,12 @@ ogmrip_x264_set_property (GObject *gobject, guint property_id, const GValue *val
     case PROP_GLOBAL_HEADER:
       x264->priv->global_header = g_value_get_boolean (value);
       break;
+    case PROP_PSY_RD:
+      x264->priv->psy_rd = g_value_get_double (value);
+      break;
+    case PROP_PSY_TRELLIS:
+      x264->priv->psy_trellis = g_value_get_double (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
       break;
@@ -534,16 +578,19 @@ ogmrip_x264_set_quality (OGMRipVideoCodec *video, OGMRipQualityType quality)
         x264->priv->brdo = TRUE;
         x264->priv->x88dct = TRUE;
         x264->priv->mixed_refs = TRUE;
+        x264->priv->psy_rd = 1.0;
         break;
       case OGMRIP_QUALITY_HIGH:
         x264->priv->brdo = FALSE;
         x264->priv->x88dct = TRUE;
         x264->priv->mixed_refs = TRUE;
+        x264->priv->psy_rd = 0.0;
         break;
       case OGMRIP_QUALITY_NORMAL:
         x264->priv->brdo = FALSE;
         x264->priv->x88dct = FALSE;
         x264->priv->mixed_refs = FALSE;
+        x264->priv->psy_rd = 0.0;
         break;
     }
   }
@@ -578,6 +625,43 @@ static OGMRipVideoPlugin x264_plugin =
   16
 };
 
+static gboolean
+ogmrip_x264_check_option (const gchar *option)
+{
+  GPtrArray *argv;
+  gint status = 1;
+
+  argv = g_ptr_array_new ();
+
+  g_ptr_array_add (argv, g_strdup ("mencoder"));
+  g_ptr_array_add (argv, g_strdup ("-nocache"));
+  g_ptr_array_add (argv, g_strdup ("-nosound"));
+  g_ptr_array_add (argv, g_strdup ("-really-quiet"));
+  g_ptr_array_add (argv, g_strdup ("-frames"));
+  g_ptr_array_add (argv, g_strdup ("0"));
+  g_ptr_array_add (argv, g_strdup ("-rawvideo"));
+  g_ptr_array_add (argv, g_strdup ("pal:fps=2"));
+  g_ptr_array_add (argv, g_strdup ("-demuxer"));
+  g_ptr_array_add (argv, g_strdup ("rawvideo"));
+  g_ptr_array_add (argv, g_strdup ("-o"));
+  g_ptr_array_add (argv, g_strdup ("/dev/null"));
+  g_ptr_array_add (argv, g_strdup ("-ovc"));
+  g_ptr_array_add (argv, g_strdup ("x264"));
+  g_ptr_array_add (argv, g_strdup ("-x264encopts"));
+  g_ptr_array_add (argv, g_strdup_printf ("%s:bitrate=800", option));
+  g_ptr_array_add (argv, g_strdup ("/dev/zero"));
+  g_ptr_array_add (argv, NULL);
+
+  g_spawn_sync (NULL, (gchar **) argv->pdata, NULL,
+      G_SPAWN_SEARCH_PATH | G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL,
+      NULL, NULL, NULL, NULL, &status, NULL);
+  
+  g_ptr_array_free (argv, TRUE);
+
+  return status == 0;
+
+}
+
 OGMRipVideoPlugin *
 ogmrip_init_plugin (void)
 {
@@ -596,6 +680,10 @@ ogmrip_init_plugin (void)
   if (!match)
     return NULL;
 
+  x264_have_brdo = ogmrip_x264_check_option ("brdo");
+  x264_have_bime = ogmrip_x264_check_option ("bime");
+  x264_have_psy  = ogmrip_x264_check_option ("psy_rd=1,1");
+
   x264_plugin.type = OGMRIP_TYPE_X264;
 
   return &x264_plugin;
