@@ -1,5 +1,5 @@
 /* OGMRip - A DVD Encoder for GNOME
- * Copyright (C) 2004-2008 Olivier Rolland <billl@users.sf.net>
+ * Copyright (C) 2004-2009 Olivier Rolland <billl@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -2328,7 +2328,7 @@ ogmrip_main_about_activated (OGMRipData *data)
 
   const gchar *authors[] = 
   { 
-    "Olivier Rolland <billl@users.sf.net>", 
+    "Olivier Rolland <billl@users.sourceforge.net>", 
     NULL 
   };
   gchar *translator_credits = _("translator-credits");
@@ -2349,7 +2349,7 @@ ogmrip_main_about_activated (OGMRipData *data)
       "name", PACKAGE_NAME,
       "version", PACKAGE_VERSION,
       "comments", _("A DVD Encoder for GNOME"),
-      "copyright", "(c) 2004-2008 Olivier Rolland",
+      "copyright", "(c) 2004-2009 Olivier Rolland",
       "website", "http://ogmrip.sourceforge.net",
       "translator-credits", translator_credits,
       "documenters", documenters,
@@ -2976,14 +2976,22 @@ main (int argc, char *argv[])
 
   data = ogmrip_main_new ();
 
-  if (argc > 1)
-    if (g_file_test (argv[1], G_FILE_TEST_EXISTS))
-      ogmrip_main_load (data, argv[1]);
-
   ogmrip_main_pref_dialog_construct (data);
   ogmrip_main_options_dialog_construct (data);
   ogmrip_main_profiles_dialog_construct (data);
 
+  if (argc > 1)
+  {
+    if (g_file_test (argv[1], G_FILE_TEST_EXISTS))
+    {
+      gchar *filename;
+
+      filename = ogmrip_fs_get_full_path (argv[1]);
+      ogmrip_main_load (data, filename);
+      g_free (filename);
+    }
+  }
+
   gtk_widget_show (data->window);
 
   gtk_main ();
