@@ -62,6 +62,7 @@ typedef struct {
   packet_t *queue_head;
   packet_t *queue_tail;
   unsigned int global_palette[16];
+  unsigned int normalized_color;
   unsigned int orig_frame_width, orig_frame_height;
   unsigned char* packet;
   size_t packet_reserve;	/* size of the memory pointed to by packet */
@@ -240,7 +241,16 @@ static void spudec_process_data(spudec_handle_t *this, packet_t *packet)
 	cmap[i] = 256 - alpha[i];
     }
     else {
+      if (this->normalize_palette)
+      {
+        if (packet->palette[i] == this->normalized_color)
+          cmap[i] = 0xff;
+        else
+          cmap[i] = 0;
+      }
+      else
       cmap[i] = ((this->global_palette[packet->palette[i]] >> 16) & 0xff);
+
       if (cmap[i] + alpha[i] > 255)
 	cmap[i] = 256 - alpha[i];
     }
@@ -367,10 +377,8 @@ static void normalize_palette(spudec_handle_t *this, packet_t *packet)
     if (packet->alpha[i] && !used[color])
     {
       used[color] = 1;
-      if (this->global_palette[color] < black)
-        this->global_palette[color] = 0;
-      else
-        this->global_palette[color] = 0xffffff;
+      if (this->global_palette[color] >= black)
+        this->normalized_color = color;
     }
   }
 }
