@@ -1,6 +1,6 @@
 /* srttool-2 - Various manipulations of srtfiles
  * Copyright (C) 2002-2006 Arne Driescher <driescher@users.sf.net>
- * Copyright (C) 2006-2008 Olivier Rolland <billl@users.sf.net>
+ * Copyright (C) 2006-2009 Olivier Rolland <billl@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -596,7 +596,7 @@ get_file_content (int fd)
 
   while (--bytes_read)
   {
-    if (isspace (buf[bytes_read]))
+    if (isspace ((int) buf[bytes_read]))
       buf[bytes_read] = '\0';
     else
       break;
